// Generated by CoffeeScript 1.8.0
var Resource, _,
  __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; },
  __slice = [].slice;

_ = require('underscore');

Resource = (function() {
  function Resource(Model, options) {
    if (options == null) {
      options = {};
    }
    if (!((_.isFunction(Model)) && (Model.modelName != null))) {
      throw new Error("Resource expected an instance of mongoose.Model");
    }
    this.Model = Model;
    this.modelName = options.modelName || this.Model.modelName.toLowerCase();
    this.modelKey = options.modelKey || ("" + this.modelName + "Id");
    this.reservedQueryParams = ['_fields', '_skip', '_limit', '_sort'];
    this.defaultPageSize = options.defaultPageSize || 10;
    this.ns = options.namespace || 'mrt';
  }

  Resource.prototype.createDoc = function(options) {
    if (options == null) {
      options = {};
    }
    return [
      this.namespace(), this.create(), this.publish({
        statusCode: 201
      })
    ];
  };

  Resource.prototype.readDoc = function(options) {
    if (options == null) {
      options = {};
    }
    return [
      this.namespace(), this.read(), this.fields(), this.execute(), this.publish({
        statusCode: 200
      })
    ];
  };

  Resource.prototype.patchDoc = function(options) {
    if (options == null) {
      options = {};
    }
    return [
      this.namespace(), this.read(), this.execute(), this.patch(), this.publish({
        statusCode: 200
      })
    ];
  };

  Resource.prototype.putDoc = function(options) {
    if (options == null) {
      options = {};
    }
    return [
      this.namespace(), this.read(), this.execute(), this.put(), this.publish({
        statusCode: 200
      })
    ];
  };

  Resource.prototype.removeDoc = function(options) {
    if (options == null) {
      options = {};
    }
    return [
      this.namespace(), this.read(), this.execute(), this.remove(), this.publish({
        statusCode: 204,
        empty: true
      })
    ];
  };

  Resource.prototype.readDocs = function(options) {
    if (options == null) {
      options = {};
    }
    return [
      this.namespace(), this.readAll(), this.pagination(), this.filters(), this.sort(), this.fields(), this.execute(), this.publish({
        statusCode: 200
      })
    ];
  };

  Resource.prototype.patchDocs = function(options) {
    if (options == null) {
      options = {};
    }
    return [
      this.namespace(), this.readAll(), this.pagination(), this.filters(), this.sort(), this.patchAll(), this.publish({
        statusCode: 200
      })
    ];
  };

  Resource.prototype.removeDocs = function(options) {
    if (options == null) {
      options = {};
    }
    return [
      this.namespace(), this.readAll(), this.pagination(), this.filters(), this.sort(), this.removeAll(), this.publish({
        statusCode: 200,
        empty: true
      })
    ];
  };

  Resource.prototype.countDocs = function(options) {
    if (options == null) {
      options = {};
    }

    /*
        Counts the documents from the current collection which match the
        optional filters
     */
    return [
      this.namespace(), this.readAll(), this.filters(), this.countAll(), this.publish({
        statusCode: 200
      })
    ];
  };

  Resource.prototype.namespace = function(options) {
    return (function(_this) {
      return function(req, res, next) {
        req[_this.ns] = {};
        return next();
      };
    })(this);
  };

  Resource.prototype.read = function(options) {
    if (options == null) {
      options = {};
    }
    return (function(_this) {
      return function(req, res, next) {
        var id;
        id = req.params[_this.modelKey];
        if (id == null) {
          return res.status(404).send({
            msg: "Document id not provided"
          });
        }
        req[_this.ns].query = _this.Model.findOne().where('_id').equals(id);
        return next();
      };
    })(this);
  };

  Resource.prototype.create = function(options) {
    if (options == null) {
      options = {};
    }
    return (function(_this) {
      return function(req, res, next) {
        var document;
        document = new _this.Model(req.body);
        return document.save(function(errors) {
          if (errors != null) {
            return res.status(500).send({
              msg: 'Error storing new document'
            });
          }
          req[_this.ns].result = document;
          return next();
        });
      };
    })(this);
  };

  Resource.prototype.patch = function(options) {
    if (options == null) {
      options = {};
    }
    return (function(_this) {
      return function(req, res, next) {
        var newValue, path, _ref;
        _ref = req.body;
        for (path in _ref) {
          newValue = _ref[path];
          req[_this.ns].result.set(path, newValue);
        }
        return req[_this.ns].result.save(function(error) {
          if (error != null) {
            return res.status(500).send({
              msg: "Error patching document"
            });
          }
          return next();
        });
      };
    })(this);
  };

  Resource.prototype.put = function(options) {
    if (options == null) {
      options = {};
    }
    return (function(_this) {
      return function(req, res, next) {
        return req[_this.ns].result.remove(function(error) {
          var document;
          if (error != null) {
            return res.status(500).send({
              msg: "Failed to replace the document"
            });
          }
          document = new _this.Model(req.body);
          document._id = req[_this.ns].result._id;
          return document.save(function(error) {
            if (error != null) {
              return res.status(500).send({
                msg: "Error replacing document"
              });
            }
            req[_this.ns].result = document;
            return next();
          });
        });
      };
    })(this);
  };

  Resource.prototype.remove = function(options) {
    if (options == null) {
      options = {};
    }
    return (function(_this) {
      return function(req, res, next) {
        return req[_this.ns].result.remove(function(error) {
          if (error != null) {
            return res.status(500).send({
              msg: 'error removing document'
            });
          }
          return next();
        });
      };
    })(this);
  };

  Resource.prototype.readAll = function(options) {
    if (options == null) {
      options = {};
    }
    return (function(_this) {
      return function(req, res, next) {
        req[_this.ns].query = _this.Model.find();
        return next();
      };
    })(this);
  };

  Resource.prototype.patchAll = function(options) {
    if (options == null) {
      options = {};
    }
    return (function(_this) {
      return function(req, res, next) {
        req[_this.ns].query.setOptions({
          multi: true
        });
        return req[_this.ns].query.update(req.body, function(error) {
          if (error != null) {
            return res.status(500).send({
              msg: 'Unable to patch selected documents'
            });
          }
          return next();
        });
      };
    })(this);
  };

  Resource.prototype.removeAll = function(options) {
    if (options == null) {
      options = {};
    }
    return (function(_this) {
      return function(req, res, next) {
        return req[_this.ns].query.remove(function(error, numDocsRemoved) {
          if (error != null) {
            return res.status(500).send({
              msg: 'Failed to remove selected documents'
            });
          }
          return next();
        });
      };
    })(this);
  };

  Resource.prototype.updateAll = function(options) {
    if (options == null) {
      options = {};
    }
  };

  Resource.prototype.countAll = function(options) {
    if (options == null) {
      options = {};
    }
    return (function(_this) {
      return function(req, res, next) {
        return req[_this.ns].query.count(function(error, count) {
          if (error != null) {
            return res.status(500).send({
              msg: 'Error counting documents'
            });
          }
          req[_this.ns].result = count;
          return next();
        });
      };
    })(this);
  };

  Resource.prototype.execute = function(options) {
    if (options == null) {
      options = {};
    }
    return (function(_this) {
      return function(req, res, next) {
        return req[_this.ns].query.exec(function(error, documents) {
          if (error != null) {
            return res.status(500).send({
              msg: error.message
            });
          }
          if (documents == null) {
            return res.status(404).send({
              msg: 'Resources not found'
            });
          }
          req[_this.ns].result = documents;
          return next();
        });
      };
    })(this);
  };

  Resource.prototype.publish = function(options) {
    if (options == null) {
      options = {};
    }
    return (function(_this) {
      return function(req, res, next) {
        var output;
        if (options.statusCode == null) {
          options.statusCode = 200;
        }
        res.status(options.statusCode);
        if (options.empty == null) {
          options.empty = false;
        }
        if (options.empty === true) {
          return res.send();
        }
        output = {
          meta: _.clone(req.query),
          data: _this.format(req[_this.ns].result)
        };
        return res.send(output);
      };
    })(this);
  };

  Resource.prototype.fields = function(options) {
    if (options == null) {
      options = {};
    }
    return (function(_this) {
      return function(req, res, next) {
        var fields;
        if (req.query._fields == null) {
          return next();
        }
        fields = (req.query._fields.split(',')).join(' ');
        req[_this.ns].query.select(fields);
        return next();
      };
    })(this);
  };

  Resource.prototype.pagination = function(options) {
    if (options == null) {
      options = {};
    }
    return (function(_this) {
      return function(req, res, next) {
        var limit, skip;
        skip = req.query._skip != null ? req.query._skip : 0;
        limit = req.query._limit != null ? req.query._limit : _this.defaultPageSize;
        req[_this.ns].query.skip(skip);
        req[_this.ns].query.limit(limit);
        return next();
      };
    })(this);
  };

  Resource.prototype.filters = function(options) {
    if (options == null) {
      options = {};
    }
    return (function(_this) {
      return function(req, res, next) {
        var key, operator, param, value, where, _ref;
        _ref = req.query;
        for (key in _ref) {
          value = _ref[key];
          if (!(!(__indexOf.call(_this.reservedQueryParams, key) >= 0))) {
            continue;
          }
          if (key.slice(-4) === '__eq') {
            where = key.slice(0, -4);
            operator = 'equals';
            param = value;
          } else if (key.slice(-4) === '__ne') {
            where = key.slice(0, -4);
            operator = 'ne';
            param = value;
          } else if (key.slice(-4) === '__gt') {
            where = key.slice(0, -4);
            operator = 'gt';
            param = +value;
          } else if (key.slice(-5) === '__gte') {
            where = key.slice(0, -5);
            operator = 'gte';
            param = +value;
          } else if (key.slice(-4) === '__lt') {
            where = key.slice(0, -4);
            operator = 'lt';
            param = +value;
          } else if (key.slice(-5) === '__lte') {
            where = key.slice(0, -5);
            operator = 'lte';
            param = +value;
          } else if (key.slice(-4) === '__in') {
            where = key.slice(0, -4);
            operator = 'in';
            param = value.split(',');
          } else if (key.slice(-7) === '__regex') {
            where = key.slice(0, -7);
            operator = 'regex';
            param = RegExp("" + value, "gim");
          } else {
            where = key;
            operator = 'equals';
            param = value;
          }
          req[_this.ns].query.where(where)[operator](param);
        }
        return next();
      };
    })(this);
  };

  Resource.prototype.sort = function(options) {
    if (options == null) {
      options = {};
    }
    return (function(_this) {
      return function(req, res, next) {
        if (_.isString(req.query._sort)) {
          req[_this.ns].query.sort(req.query._sort);
        }
        return next();
      };
    })(this);
  };

  Resource.prototype.log = function() {
    var context, level, message;
    level = arguments[0], message = arguments[1], context = 3 <= arguments.length ? __slice.call(arguments, 2) : [];
    return console.log.apply(console, [level, message].concat(__slice.call(context)));
  };

  Resource.prototype.format = function(results) {
    return results;
  };

  return Resource;

})();

module.exports = Resource;
